var Game = {
  
  debugMode : false,
  
  bookMoney : function(amount, source) {
    if(amount < 0) {
      if(!Game.state.bank.minus[source])
        Game.state.bank.minus[source] = 0;
      Game.state.bank.minus[source] += amount;
    } else {
      if(!Game.state.bank.plus[source])
        Game.state.bank.plus[source] = 0;
      Game.state.bank.plus[source] += amount;
    }
    Game.state.money += amount;
    Game.state.yikes.score += amount;
  },
  
  getMoneyReport : function() {
    var items = {};
    var report = [];
    each(Game.state.bank.plus, function(amount, key) {
      items[key] = amount;
    });
    each(Game.state.bank.minus, function(amount, key) {
      if(!items[key])
        items[key] = 0;
      items[key] += amount;
    });
    each(items, function(amount, key) {
      report.push(Units.buildings[key].name+'s: '+(Math.round(amount*10)/10)+'M');
    });
    return(report.join('<br/>'));
  },
  
  seasonLength : 600,
  
  seasonEnd : function() {
    Game.paused = true;
    UI.showWindow(
      'THE SEASON HAS ENDED!<br/><br/>'+
      'Your resort has <em>'+Math.round(Game.state.yikes.rating)+'</em> stars<br/>'+
      '<em>'+(Game.state.leftCtr)+'</em> people completed their vacation<br/>'+
      'The overall score is <em>'+Math.round(Game.state.yikes.score)+'</em> <br/>'+
      'Your bank account: <em>$'+Math.round(Game.state.money)+'M</em>',
      function() {
        Game.startNew();
      }
      );
  },
  
  state : {
    map : {},
  },
  
  init : function() {
  },
  
  save : function() {
    localStorage.savegame = JSON.stringify(Game.state);
  },
  
  clear : function() {
    Game.state = {
      map : {},
      people : [],
    };
    Units.clear();
    GameMap.clear();
    People.clear();
  },
  
  startNew : function() {
    Game.clear();
    GameMap.createNew(18, 10, [
        '                    ',
        '    gggggggg        ',
        ' gggggggggggggggg   ',
        '    gggggggggggggg  ',
        '   ggggggggggggg    ',
        '    gggg  ggggggg   ',
        '  ggg      ggggg    ',
        '  gggg      ggg     ',
      ]);  
    Game.instantiate();
    Game.state.money = 10;
    Game.state.buildingCapacity = Units.buildingCapacity;
    Game.state.seasonLength = Game.seasonLength;
    UI.showWindow('Welcome to your new job, [INSERTNAMEHERE]<br/><br/>'+
      'This is your island to manage for the season. As you can see, there is nothing '+
      'on it yet. We at the Small World Resort franchise headquarters are counting on you! '+
      'Build the necessary things that tourists might enjoy and get your '+
      'parent company\'s bosses a nice bonus by making sure we get a good rating!<br/><br/>'+
      'Good luck!');
  },
  
  load : function() {
    if(localStorage.savegame && localStorage.savegame != '') {
      Game.clear();
    	Game.state = JSON.parse(localStorage.savegame);
    	Game.instantiate();
    } else {
      Game.startNew();
    }
  },
  
  paused : false,
  
  research : {
    
    capacity : {
      name : 'Increased Capacity',
      text : 'Completing this research will double the capacity of your buildings!',
      cost : 1,
      onComplete : function() {
        Game.state.buildingCapacity = Units.buildingCapacity*2;
      },
    },
    
    buzz : {
      name : 'Social Media',
      text : 'Completing this research will double your buzz!',
      cost : 2,
      onComplete : function() {
        Game.state.yikes.buzzFactor = 2;
        Game.state.yikes.buzz *= 2;
      },
    },
    
    money : {
      name : 'Shady Credits',
      text : 'Completing this research will give you $20M!',
      cost : 0,
      onComplete : function() {
        Game.state.money += 20;
        UI.message('Received $20M');
      },
    },
    
    extension : {
      name : 'Extended Season',
      text : 'Completing this research will double the duration of the season!',
      cost : 10,
      onComplete : function() {
        Game.state.buildingCapacity = Units.buildingCapacity*2;
      },
    },
    
  },
  
  startResearch : function(key) {
    UI.closeWindow();
    var rSpec = Game.research[key];
    if(Game.state.research.current) {
      UI.message('Must finish current research first');
      return;
    } else if(Game.state.money < rSpec.cost) {
      UI.message('Cannot afford to research '+rSpec.name+' now.');
      return;
    } else if(Game.state.research[key].completed) {
      UI.message('Already researched '+rSpec.name+'.');
      return;
    } else {
      Game.state.research.current = {
        topic : key,
        startTime : Game.state.time,
        endTime : Game.state.time + 30,    
        duration : 30,    
      }
      if(rSpec.cost > 0)
        Game.bookMoney(-rSpec.cost, 'research');
      UI.message('Started research: '+rSpec.name);
    }
  },
  
  finishResearch : function() {
    if(Game.state.research.current) {
      var key = Game.state.research.current.topic;
      Game.state.research.current.completed = true;
      Game.state.research[key] = Game.state.research.current;
      Game.state.research.current = false;
      var rSpec = Game.research[key];
      if(rSpec.onComplete)
        rSpec.onComplete();
      UI.message('Research done: '+rSpec.name);
    }
  },
  
  lastResearchTick : 0,
  researchTick : function() {
    if(Game.lastResearchTick < Game.state.time - 1) {
      Game.lastResearchTick = Game.state.time; 
      if(Game.state.research && Game.state.research.current) {
        var rSpec = Game.research[Game.state.research.current.topic];
        var r = Game.state.research.current;
        $('#research-bar').css('display', 'inherit');
        $('#research-bar .text').text(rSpec.name);
        $('#research-bar .bar').css('width', (300*(Game.state.time-r.startTime)/r.duration)+'px');
        if(Game.state.time > Game.state.research.current.endTime) {
          Game.finishResearch();
        }
      } else {
        $('#research-bar').css('display', 'none');
      }
    }
  },
  
  instantiate : function() {
    Game.paused = false;
  	if(!Game.state.people)
  	  Game.state.people = [];
    if(!Game.state.time)
      Game.state.time = 1;
    if(!Game.state.idCtr)
      Game.state.idCtr = 1;
    if(!Game.state.leftCtr)
      Game.state.leftCtr = 0;
    if(!Game.state.arrivedCtr)
      Game.state.arrivedCtr = 0;
    if(!Game.state.research)
      Game.state.research = {};
    if(!Game.state.yikes)
      Game.state.yikes = {
        needs : {},
        frustrations : {},
        rating : 2,
        buzz : 5,
        score : 0,
        buzzFactor : 1,
      };
    if(!Game.state.bank)
      Game.state.bank = {
        plus : {},
        minus : {},
      };
    if(!Game.state.money)
      Game.state.money = 0;
    GameMap.instantiate(); 
    People.instantiate();
  },
  
}