var GameMap = {
  
  init : function() {
    GameMap.container = new PIXI.Container();
    Stage.add(GameMap.container);
    GameMap.containerOverlay = new PIXI.Container();
    Stage.add(GameMap.containerOverlay);
    GameMap.initCursor();
  },
  
  cursorOnTileObject : false,
  initCursor : function() {
    GameMap.cursor = new PIXI.Graphics();
    GameMap.cursor.lineStyle(1, 0xffffff, 1.0);
    GameMap.cursor.scale = new PIXI.Point(1, 1);
    GameMap.cursor.drawRect(0, 0, Stage.tileSize, Stage.tileSize);
    GameMap.containerOverlay.addChild(GameMap.cursor);
    GameMap.cursor.data = {
      x : 5,
      y : 5,
      };
    Stage.animation(function() {
      GameMap.containerOverlay.addChild(GameMap.cursor); // WTF?
      if(GameMap.cursor.data.x != Math.floor(Stage.mouse.x) || 
        GameMap.cursor.data.y != Math.floor(Stage.mouse.y)) {
          GameMap.cursor.data.x = Math.floor(Stage.mouse.x);      
          GameMap.cursor.data.y = Math.floor(Stage.mouse.y);    
          Stage.updatePosition(GameMap.cursor);
          GameMap.cursorOnTileObject = GameMap.getTileGameObjectAt(GameMap.cursor.data.x, GameMap.cursor.data.y);
          if(GameMap.cursorOnTileObject &&
            UI.activePaletteItem.canDo && 
            UI.activePaletteItem.canDo(GameMap.cursorOnTileObject.data)) {
            GameMap.cursor.tint = 0xffffff;
          } else {
            GameMap.cursor.tint = 0xff4444;
          }
        }
      return(true);
    });
    Stage.queue(function() { GameMap.cursor.cacheAsBitmap = false; });
  },

  clearTile : function(pos) {
    var x = Math.round(pos.x);
    var y = Math.round(pos.y);
    var tile = GameMap.getTileGameObjectAt(x, y);
    if(tile) {
      //console.log('clear tile', x, y);
      if(tile.data.building) {
        Units.destroyBuildingOnTile(tile);
      }
    }
  },
  
  actionClick : function(pos) {
    if(UI.activePaletteItem) {
      //console.log('action', UI.activePaletteItem.key);
      UI.paletteAction(pos);
    }
  },

  createNew : function(width, height, fromText) {
    //console.log('GameMap.createNew('+width+', '+height+')');
    Game.state.map = { 
      tiles : {}, 
      width : width, 
      height : height };
    for(var x = 0; x < Game.state.map.width; x += 1) {
      for(var y = 0; y < Game.state.map.height; y += 1) {
        var tile = {
          x : x,
          y : y,
          type : 'ground',
          building : false,
          };
        if(fromText) {
          if(fromText[y]) {
            var c = fromText[y].substr(x, 1);
            if(c == ' ' | c == '' || !c) {
              tile.type = 'water';
            }
          } else {
            tile.type = 'water';
          }
        }
        Game.state.map.tiles[x+':'+y] = tile;
      }
    }
  },
  
  bgLookup : {
    water : 0x3399ff,
    ground : 0x33ff55,
  },
  
  doNeighbors(tile, f) {
    for(var ix = -1; ix < 2; ix += 1) {
      for(var iy = -1; iy < 2; iy += 1) {
        var t = GameMap.getTileGameObjectAt(ix+tile.x, iy+tile.y);
        if(t) 
          f(t.data);
      }
    }
  },
  
  tileIsOnCoast(tile) {
    var foundGround = false;
    var foundWater = false;
    GameMap.doNeighbors(tile, function(t) {
      if(t.type == 'ground')
        foundGround = true;
      else if(t.type == 'water')
        foundWater = true;
    });
    //console.log('tile on coast', foundGround, foundWater);
    return(foundGround && foundWater);
  },
  
  getTile : function(x, y) {
    return(Game.state.map.tiles[x+':'+y] || {});
  },
  
  draw : function(o) {
    o.clear();
    o.removeChildren();
    o.cacheAsBitmap = false;
    o.beginFill(GameMap.bgLookup[o.data.type]);
    o.drawRect(0, 0, Stage.tileSize, Stage.tileSize);
    o.beginFill(0x000000);
    o.drawRect(0, 0, 1, 1);
    if(o.data.type == 'ground') {
      if(GameMap.getTile(o.data.x+1, o.data.y+1).type == 'water') {
        o.beginFill(GameMap.bgLookup['water']);
        o.drawRect(Stage.tileSize-4, Stage.tileSize-4, 4, 4);
      }
      if(GameMap.getTile(o.data.x-1, o.data.y+1).type == 'water') {
        o.beginFill(GameMap.bgLookup['water']);
        o.drawRect(0, Stage.tileSize-4, 4, 4);
      }
      if(GameMap.getTile(o.data.x-1, o.data.y-1).type == 'water') {
        o.beginFill(GameMap.bgLookup['water']);
        o.drawRect(0, 0, 4, 4);
      }
      if(GameMap.getTile(o.data.x+1, o.data.y-1).type == 'water') {
        o.beginFill(GameMap.bgLookup['water']);
        o.drawRect(Stage.tileSize-4, 0, 4, 4);
      }
    }
    if(o.data.building) {
      Units.updateBuildingGraphics(o);
    }
    Stage.queue(function() { o.cacheAsBitmap = true; });
  },
  
  getTileGameObjectAt : function(x, y) {
    return(GameMap.tileIndex[x+':'+y]);
  },
  
  clear : function() {
    GameMap.container.removeChildren();
    GameMap.containerOverlay.removeChildren();
  },
  
  pathGrid : {},
  pathFinder : new PF.AStarFinder(),
  updatePathfinding : function() {
    GameMap.pathGrid = new PF.Grid(Game.state.map.width, Game.state.map.height);
    each(Game.state.map.tiles, function(tile) {
      GameMap.pathGrid.setWalkableAt(tile.x, tile.y, tile.type == 'ground');
      });
  },
  findPath : function(fromX, fromY, toX, toY) {
    return(GameMap.pathFinder.findPath(fromX, fromY, toX, toY, GameMap.pathGrid.clone()));
  },
  
  tileIndex : {},
  container : false,
  containerOverlay : false,
  instantiate : function() {
    GameMap.tileIndex = {};
    each(Game.state.map.tiles, function(tile) {
      var tileGameObject = new PIXI.Graphics({ });
      GameMap.tileIndex[tile.x+':'+tile.y] = tileGameObject;
      tileGameObject.data = tile;
      GameMap.draw(tileGameObject);
      Stage.updatePosition(tileGameObject);
      GameMap.container.addChild(tileGameObject);
    });
    GameMap.updatePathfinding();
  },
  
}