var People = {
  
  byId : {},
  
  init : function() {
    People.container = new PIXI.Container();
    Stage.add(People.container);
  },
  
  instantiate : function() {
    each(Game.state.people, function(person) {
      if(!person.id)
        person.id = Game.state.idCtr++;
      person.xOffset = Math.random()*Stage.tileSize - Stage.tileSize/2;
      person.yOffset = Math.random()*Stage.tileSize - Stage.tileSize/2;
      People.initNeeds(person);
      People.spawnGameObject(person);
    });
  },
  
  clear : function() {
    People.container.removeChildren();
    People.byId = {};
  },
  
  draw : function(go) {
    go.cacheAsBitmap = false;
    go.clear();
    go.beginFill(0xaa7755, 1);
    go.drawCircle((Stage.tileSize/2), 5, 4);
    go.drawRect((Stage.tileSize/2)-3, 9, 6, 10);  
    go.scale = new PIXI.Point(0.8, 0.8);  
    Stage.queue(function() { go.cacheAsBitmap = true; });
  },
  
  spawnGameObject : function(person) {
    var go = new PIXI.Graphics();
    People.byId[person.id] = go;
    go.data = person;
    People.draw(go);
    People.container.addChild(go);
    Stage.updatePosition(go);
  },
  
  leave : function(person) {
    var go = People.byId[person.id];
    People.container.removeChild(go);
    Game.state.leftCtr += 1;
    People.leaveFromLocation(person);
    removeItem(Game.state.people, person);
    Yikes.recordSentiment(person, 0.1);
    console.log('person', person.id, 'leaves');
    Sound.play('bye');
  },

  spawn : function(pos) {
    var person = {
      x : pos.x,
      y : pos.y,
      id : Game.state.idCtr++,
    };
    Game.state.arrivedCtr += 1;
    People.initNeeds(person);
    Game.state.people.push(person);
    People.spawnGameObject(person);
    Sound.play('woohoo');
    UI.plop(pos.x, pos.y, '$.02M', 0x00ff00);
    Game.state.money += 0.02;
  },
  
  needs : {
    home : {
      name : 'Bye',
      linkToBuilding : 'harbor',
      buildupSpeed : 0.1,
      startsAt : 0,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('harbor')));
      },
    },
    hunger : {
      name : 'Hunger',
      linkToBuilding : 'restaurant',
      buildupSpeed : 0.2,
      startsAt : 0.6,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('restaurant')));
      },
    },
    hiking : {
      name : 'Hiking',
      buildupSpeed : 0.1,
      startsAt : 0,
      action : function(person) {
        var groundWithoutBuilding = filter(Game.state.map.tiles, function(t) {
          return(t.type == 'ground' && !t.building);
        });
        var hikeToTile = randomItem(groundWithoutBuilding);
        if(hikeToTile) {
          People.walkTo(person, hikeToTile.x, hikeToTile.y);
          People.updateNeed(person, 'hiking', -10000);
          return(true);
        }
      },
    },
    beach : {
      name : 'Beach',
      linkToBuilding : 'beach',
      buildupSpeed : 0.4,
      startsAt : 0,
      seed : 10,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('beach')));
      },
    },
    pool : {
      name : 'Pool',
      linkToBuilding : 'pool',
      buildupSpeed : 0.2,
      startsAt : 0.4,
      seed : 2,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('pool')));
      },
    },
    party : {
      name : 'Party',
      linkToBuilding : 'club',
      buildupSpeed : 0.3,
      startsAt : 0.7,
      seed : 5,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('club')));
      },
    },
    rest : {
      name : 'Rest',
      linkToBuilding : 'hotel',
      buildupSpeed : 0.2,
      startsAt : 0.2,
      action : function(person) {
        return(People.useBuilding(person, Units.findA('hotel')));
      },
    },
  },
  
  updateNeed : function(person, needKey, modValue) {
    person.needs[needKey].current += modValue;
    if(person.needs[needKey].current < 0)
      person.needs[needKey].current = 0;
  },
  
  initNeeds : function(person) {
    if(!person.needs)
      person.needs = {};
    each(People.needs, function(need, key) {
      if(!person.needs[key]) person.needs[key] = { 
        current : Math.random(),
        preference : (1+Math.random()+Math.random())/2,
        frustration : 0,
        };
      if(need.seed)
        person.needs[key].current = need.seed;
    });
  },
  
  tickNeeds : function(person) {
    var highestNeedKey = false;
    var highestNeedValue = 0;
    each(person.needs, function(need, key) {
      var nSpec = People.needs[key];
      if(!nSpec) {
        console.log('error: need not specified', key);          
      } else {
        if(Game.state.seasonProgress < nSpec.startsAt)
          return;
        need.current += nSpec.buildupSpeed*need.preference;
        if(need.current > highestNeedValue) {
          highestNeedValue = need.current;
          highestNeedKey = key;
        }
      }
    });
    if(highestNeedValue > 10 && !person.task) { 
      // go, pursue that need
      var nSpec = People.needs[highestNeedKey];
      if(nSpec.action(person)) {
        UI.speechBubble(person.x, person.y, nSpec.name, 0x000000);
      } else {
        UI.speechBubble(person.x, person.y, nSpec.name, 0xaa0000);
        person.needs[highestNeedKey].current = 0;          
        person.needs[highestNeedKey].frustration += 3*Math.random();  
        UI.plop(person.x, person.y, ':(', 0xff0000);
        person.needs.home.current += 3;
        Sound.play('meh');    
        UI.message('No '+nSpec.linkToBuilding+' available', '#ffaa88');    
      }
    }
    Yikes.recordSentiment(person); 
  },

  useBuilding : function(person, buildingTile) {
    if(!buildingTile) {
      return;
    }
    People.walkTo(person, buildingTile.x, buildingTile.y);
    person.wantsToUse = buildingTile.building.type;
    return(true);
  },
  
  walkTo : function(person, x, y) {
    var path = GameMap.findPath(person.x, person.y, x, y);
    if(path.length == 0) {
      console.log('no path from', person.x, person.y, 'to', x, y);      
      return(false);
    }
    People.leaveFromLocation(person);
    person.task = {
      type : 'walk',
      duration : 1,
      startTime : Game.state.time,
      waypoints : path,
      };
  },
  
  doStuff : function(person) {
    var lx = Math.floor(person.x);
    var ly = Math.floor(person.y);
    var tile = Game.state.map.tiles[lx+':'+ly];
    if(tile.building) {
      var bSpec = Units.buildings[tile.building.type];
      if(bSpec.onUse)
        bSpec.onUse(tile, person);
    }
  },
  
  arriveAtLocation : function(person) {
    var x = Math.floor(person.x);
    var y = Math.floor(person.y);
    //console.log(person.id, 'arrives at', x, y);
    if(!Game.state.map.tiles[x+':'+y].people)
      Game.state.map.tiles[x+':'+y].people = [];
    Game.state.map.tiles[x+':'+y].people.push(person.id);
    if(Game.state.map.tiles[x+':'+y].people.length >= Game.state.buildingCapacity) {
      UI.plop(x, y, 'full', 0xff0000);
      UI.message(Game.state.map.tiles[x+':'+y].building.type+' is full!', '#ffaa88');
    }
    People.doStuff(person);
  },
  
  leaveFromLocation : function(person) {
    var x = Math.floor(person.x);
    var y = Math.floor(person.y);
    //console.log(person.id, 'leaves', x, y);
    if(!Game.state.map.tiles[x+':'+y].people)
      return;
    removeItem(Game.state.map.tiles[x+':'+y].people, person.id);
  },
  
  tasks : {
    
    walk : function(person) {
      var ctime = Game.state.time - person.task.startTime;

      if(ctime >= person.task.duration) {
        ctime = 0;
        person.task.startTime = Game.state.time;
        person.task.waypoints.shift();
      }

      if(person.task.waypoints.length <= 1) {
        if(person.task.waypoints.length == 1) {
          person.x = person.task.waypoints[0][0];
          person.y = person.task.waypoints[0][1];
        }
        person.x = Math.floor(person.x);
        person.y = Math.floor(person.y);
        delete person.task;
        People.arriveAtLocation(person);
        return;
      }
      
      person.x = lerp(person.task.waypoints[0][0], person.task.waypoints[1][0], ctime/person.task.duration);
      person.y = lerp(person.task.waypoints[0][1], person.task.waypoints[1][1], ctime/person.task.duration);
    },
    
  },
  
  tick : function(deltaTime) {
    each(Game.state.people, function(person) {
      if(!person) return;
      if(!person.lastTickTime)
        person.lastTickTime = Game.state.time;
      if(person.task) {
        var taskHandler = People.tasks[person.task.type];
        if(taskHandler)
          taskHandler(person);
        Stage.updatePosition(People.byId[person.id]);
      } else if(person.lastTickTime < Game.state.time - 1) {
        People.doStuff(person);
        People.tickNeeds(person);
        person.lastTickTime = Game.state.time;
      }
    });
  },
  
}