var Stage = {
  
  tileSize : 32,
  statsElement : false,
  init : function(width, height) {
    PIXI.SCALE_MODES.DEFAULT = PIXI.SCALE_MODES.NEAREST;
    Stage.renderer = PIXI.autoDetectRenderer(width, height, {
      antialias: false, 
      transparent: false, 
      autoResize : true,
      backgroundColor : GameMap.bgLookup['water'],
      });
    window.onresize = function (event) {    
      var w = window.innerWidth;    
      var h = window.innerHeight;    
      Stage.renderer.view.style.width = w + "px";    
      Stage.renderer.view.style.height = h + "px";    
      Stage.renderer.resize(w,h);
      }
    Stage.statsElement = $('#render-stats');
    //Stage.renderer.view.style.transform = 'scale(3,3)';
    document.getElementById('stage-container').appendChild(Stage.renderer.view);
    Stage.root = new PIXI.Container();
    Stage.root.scale = new PIXI.Point(2, 2);   
    Stage.root.interactive = true;
    $('#stage-container').on('mousemove', function(e) {
      var pt = Stage.root.toLocal({ x : e.clientX, y : e.clientY});
      Stage.mouse.x = pt.x / Stage.tileSize;
      Stage.mouse.y = pt.y / Stage.tileSize;
      Stage.mouse.rawX = e.clientX;
      Stage.mouse.rawY = e.clientY;
      if(Stage.mouse.button && (Math.abs(Stage.mouse.rawX - Stage.mouse.downX) > 4 || Math.abs(Stage.mouse.rawY - Stage.mouse.downY) > 4)) {
        Stage.root.x = Stage.mouse.rootX + (Stage.mouse.rawX - Stage.mouse.downX);
        Stage.root.y = Stage.mouse.rootY + (Stage.mouse.rawY - Stage.mouse.downY);
        Stage.mouse.mapDrag = true;
      }
    });
    $('#stage-container').on('mousedown', function(e) {
      Stage.mouse.mapDrag = false;
      Stage.mouse.button = true;
      Stage.mouse.downX = Stage.mouse.rawX;
      Stage.mouse.downY = Stage.mouse.rawY;
      Stage.mouse.rootX = Stage.root.x;
      Stage.mouse.rootY = Stage.root.y;
    });
    $('#stage-container').on('mouseup', function(e) {
      Stage.mouse.button = false;
      if(!Stage.mouse.mapDrag)
        GameMap.actionClick(Stage.mouse);
    });
    
    Stage.render();
    GameMap.init();
    People.init();
    UI.init();
  },

  add : function(o) {
    Stage.root.addChild(o);
  },
  
  updatePosition : function(o) {
    if(o.data) {
      o.x = Math.round(o.data.x * Stage.tileSize) + (o.data.xOffset || 0);
      o.y = Math.round(o.data.y * Stage.tileSize) + (o.data.yOffset || 0);
    }
  },
  
  queueList : [],  
  queue : function(f) {
    Stage.queueList.push(f);
  },
  
  animationList : [],
  
  lastAnimationFrame : window.performance.now() / 1000,
  deltaTime : 0,
  currentTime : 0,
  houseKeepingCounter : 0,

  animation : function(f) {
    Stage.animationList.push(f);
  },
  
  animate : function() {
    Stage.deltaTime = Stage.currentTime - Stage.lastAnimationFrame;
    Stage.houseKeepingCounter += Stage.deltaTime;
    each(Stage.animationList, function(f, idx) {
      if(f.markDeleted) 
        return;
      var result = f(Stage.deltaTime);
      if(!result)
        f.markDeleted = true;
    });
    if(Stage.houseKeepingCounter > 1) {
      Game.state.yikes.buzz -= 0.05;
      if(Game.state.yikes.buzz < 0)
        Game.state.yikes.buzz = 0;
      var cleanupCount = 0;
      Stage.animationList = filter(Stage.animationList, function(f) {
        return(!f.markDeleted);
        cleanupCount += 1;
        });
      if(cleanupCount > 0)
        console.log('animation list cleanup count: '+cleanupCount);
      Stage.statsElement.text(
        (Game.debugMode ? 
          Math.round(Stage.stats.fps)+' FPS | '+
          Math.round(Stage.stats.cpu)+'% CPU | '+
          GameMap.cursor.data.x+':'+GameMap.cursor.data.y+' | ' : '')+
        Game.state.people.length+' PPL | '+Game.state.leftCtr+' VACATIONS | '+
        stringRepeat('*', Game.state.yikes.rating)+' | '+
        'BUZZ: '+(Math.round(Game.state.yikes.buzz*10)/10)+' | '+
        'SCORE: '+Math.round(Game.state.yikes.score));
      UI.updatePalette();
      $('#status').text('Season: '+Math.round(100*Game.state.time/Game.state.seasonLength)+'% done');
      Game.state.seasonProgress = Game.state.time/Game.state.seasonLength;
      if(Game.state.time > Game.state.seasonLength) {
        Game.seasonEnd();
      }
      Stage.houseKeepingCounter = 0;
    }
  },

  tick : function() {
    if(Game.paused || UI.infoWindowIsOpen)
      return;
    Game.state.time += Stage.deltaTime;
    Units.tick(Stage.deltaTime);
    People.tick(Stage.deltaTime);
    Game.researchTick();
  },
  
  mouse : { x : 0, y : 0, button : false, downX : 0, downY : 0 },
  
  stats : { fps : 60, lastRenderFinished : 0 },
  updateStats : function() {
    Stage.stats.fps = (1 / (Stage.deltaTime))*0.02 + Stage.stats.fps*0.98;
    Stage.stats.cpu = ((Stage.stats.lastRenderFinished-Stage.currentTime)/Stage.deltaTime)*100;
  },
  
  render : function() {
    Stage.currentTime = window.performance.now() / 1000;
    Stage.tick();
    Stage.animate();
    Stage.renderer.render(Stage.root);
    Stage.stats.lastRenderFinished = window.performance.now() / 1000;
    if(Stage.queueList.length > 0) {
      each(Stage.queueList, function(f) {
        f(Stage.deltaTime);
      });
      Stage.queueList = [];
    }
    requestAnimationFrame(Stage.render);
    Stage.updateStats();
    Stage.lastAnimationFrame = Stage.currentTime;    
  },
  
}