var UI = {
  
  templates : {},
    
  init : function() {
    each(['harbor', 'beach', 'hotel', 'pool', 'restaurant', 'club'], function(pkey) {
      UI.palette[pkey] = {
        cost : Units.buildings[pkey].cost,
        name : Units.buildings[pkey].name,
        onDraw : function(pi) {
          pi.expected = false;
          var bSpec = Units.buildings[pkey];
          if(bSpec && bSpec.linkedNeed) {
            var nSpec = People.needs[bSpec.linkedNeed];
            if(nSpec)
              pi.expected = nSpec.startsAt < Game.state.seasonProgress; 
          } 
        },
        canDo : function(tile) {
          return(Units.canPlace(pkey, tile));
        },
        action : function(pos) {
          Units.constructBuildingAt(pos.x, pos.y, pkey);
        },
      };
    });
    UI.templates = {
      palette : Minibars.compile($('#palette-template').html()),
      menu : Minibars.compile($('#menu-template').html()),
      menuItem : Minibars.compile($('#menu-item-template').html()),
      researchWindow : Minibars.compile($('#research-window-template').html()),
    };    
    UI.updatePalette();  
    UI.container = new PIXI.Container();
    Stage.add(UI.container);
  },
  
  speechBubble : function(x, y, text, color, attachTo) {
    color = color || 0x000000;
    var b = new PIXI.Graphics();
    b.beginFill(0xffffff);
    b.lineStyle(1, color, 1.0);
    var tx = UI.addText(b, text, 2, 4, color);
    b.drawRoundedRect(0, 2, 4+tx.width, 10, 3);
    b.drawPolygon([ 6, 11, 4, 16, 2, 11 ]);
    if(!attachTo)
      attachTo = UI.container;
    attachTo.addChild(b);
    b.data = {
      x : x+0.2,
      y : y-0.5,
      }
    Stage.queue(function() { b.cacheAsBitmap = true; });
    Stage.updatePosition(b);
    var ttl = 1.2;
    Stage.animation(function(deltaTime) {
      ttl -= deltaTime;
      if(ttl < 0.5) {
        b.alpha = 2*ttl;
      }
      if(ttl < 0) {
        attachTo.removeChild(b);
        return(false);
      } else {
        return(true);
      }
    });
  },

  clear : function() {
    $('#message-list').html('');
  },
  
  message : function(text, color) {
    if(!Game.state.messageList)
      Game.state.messageList = [];
    if(!color)
      color = '';
    else
      color = 'color:'+color;
    Game.state.messageList.push('<div style="'+color+'">'+text+'</div>');
    while(Game.state.messageList.length > 6) {
      Game.state.messageList.shift();
    }
    $('#message-list').html(Game.state.messageList.join(''));
    Sound.pof.play();
  },
  
  infoWindowIsOpen : false,
  onInfoWindowClose : false,
  showWindow : function(text, doWhenClose) {
    UI.onInfoWindowClose = doWhenClose;
    UI.infoWindowIsOpen = true;
    $('#popup-window').css('display', 'inherit');
    $('#popup-window .content').html(text);
  },

  closeWindow : function() {
    UI.infoWindowIsOpen = false;
    $('#popup-window').css('display', 'none');
    if(UI.onInfoWindowClose)
      UI.onInfoWindowClose();
  },

  plop : function(x, y, text, color) {
    color = color || 0xffffff;
    var b = UI.addText(UI.container, text, 2, 4, color);
    b.data = {
      x : x+0.3,
      y : y+0.5,
      }
    b.scale = new PIXI.Point(0.5, 0.5);
    Stage.queue(function() { b.cacheAsBitmap = true; });
    Stage.updatePosition(b);
    var ttl = 1.2;
    Stage.animation(function(deltaTime) {
      b.y -= 10*deltaTime;
      ttl -= deltaTime;
      if(ttl < 0.5) {
        b.alpha = 2*ttl;
      }
      if(ttl < 0) {
        UI.container.removeChild(b);
        return(false);
      } else {
        return(true);
      }
    });
  },

  addText : function(o, txt, x, y, color) {
    color = color || 0x000000;
    var text = new PIXI.Text(txt.toUpperCase(), {fontFamily : 'pixel-font', fontSize: 24, fill : color, align : 'left'});
    text.scale = new PIXI.Point(0.25, 0.25);
    text.x = x || 0;
    text.y = y || 0;
    o.addChild(text);
    return(text);
  },

  activePaletteItem : false,
  paletteClick : function(key) {
    var p = UI.palette[key];
    each(UI.palette, function(pl) {
      pl.style = '';
    });
    p.style = 'highlight';
    UI.updatePalette();
    UI.activePaletteItem = p;
    Sound.click.play();
  },
  
  paletteAction : function(pos) {
    if(UI.activePaletteItem && UI.activePaletteItem.action) {
      UI.activePaletteItem.action({ x : Math.floor(pos.x), y : Math.floor(pos.y) });
    }    
  },
  
  palette : {
    bulldoze : {
      name : 'Bulldoze',
      canDo : function(tile) {
        return(tile.type !== 'water');
      },
      action : function(pos) {
        GameMap.clearTile(pos);
      },
    },
  },
  
  menuVisible : false,
  menuActive : false,
  closeMenu : function() {
    $('#menu-pane').css('display', 'none');
    UI.menuVisible = false;
    Sound.pop.play();
  },
  
  menuItemClick : function(key) {
    UI.closeMenu();
    var ma = UI.menu[UI.menuActive];
    if(ma && ma.items[key] && ma.items[key].action) {
      console.log('MENU', ma.items[key].name);
      ma.items[key].action();
    }
    Sound.click.play();
  },
  
  menuClick : function(key) {
    if(UI.menuVisible) {
      UI.closeMenu();
      return;
    }
    if(UI.menu[key].items)
      UI.execTemplate('menuItem', $('#menu-pane'), UI.menu[key].items);
    else if(UI.menu[key].onclick)
      $('#menu-pane').html(UI.menu[key].onclick());
    $('#menu-pane').css('display', 'inherit');
    UI.menuVisible = true;
    UI.menuActive = key;
    Sound.click.play();
  },
  
  menu : {
    
    money : {
      onName : function() {
        return('$'+(Math.round(Game.state.money*10)/10)+'M');
      },
      onclick : function() {
        return(
          '<div style="color:white;padding:8px;">'+
          '<div>Cash: '+'$'+(Math.round(Game.state.money*10)/10)+'M'+'</div>'+
          '<div style="opacity:0.7">'+Game.getMoneyReport()+'</div></div>');
      },
    },

    rating : {
      name : 'Ratings',
      onclick : function() {
        return(
          '<div>Stars: '+(Math.round(Game.state.yikes.rating*10)/10)+' / 5</div>'+
          '<div>Buzz: '+(Math.round(Game.state.yikes.buzz*10)/10)+' </div>'
          );
      },
    },

    research : {
      name : 'Research',
      onclick : function() {
        var params = { topics : [] };
        each(Game.research, function(rt, rkey) {
          if(!Game.state.research[rkey])
            Game.state.research[rkey] = { };
          params.topics.push({ info : rt, key : rkey, data : Game.state.research[rkey] });
        });
        UI.showWindow(UI.templates.researchWindow(params), UI.closeMenu);
        return('');
      },
    },

    game : {
      name : 'Game',
      items : {
        restart : {
          name : 'New Game',
          action : function() {
            Game.startNew();
          },
        }  
      },
    },

  },
  
  execTemplate : function(templateKey, container, dataItems, addendum) {
    var items = [];
    each(dataItems, function(pi, k) {
      pi.key = k;
      if(typeof pi.onName == 'function')
        pi.name = pi.onName();
      if(pi.onDraw)
        pi.onDraw(pi);
      items.push(UI.templates[templateKey](pi));
    });
    container.html(items.join('')+(addendum || ''));
  },
  
  updatePalette : function() {
    UI.execTemplate('palette', $('#palette'), UI.palette);
    UI.execTemplate('menu', $('#menu'), UI.menu, ' | ');
    if(Game.state.messageList)
      $('#message-list').html(Game.state.messageList.join(''));
    else
     $('#message-list').html('');
  },
  
}