var Units = {
  
  buildingList : {},
  
  buildingCapacity : 10,
  
  findA : function(type) {
    var results = [];
    each(Units.buildingList, function(bld) {
      if(bld.data.building.type == type && (!bld.data.people || bld.data.people.length < Game.state.buildingCapacity))
        results.push(bld.data);
    });
    return(randomItem(results));
  },
  
  clear : function() {
    Units.buildingList = {};
  },
  
  destroyBuildingOnTile : function(tile) {
    var go = Units.buildingList[tile.data.x+':'+tile.data.y];
    Sound.play('boom');
    tile.data.building = false;
    GameMap.draw(tile);
    delete Units.buildingList[tile.data.x+':'+tile.data.y];
  },

  updateBuildingGraphics : function(o) {
    Units.buildingList[o.data.x+':'+o.data.y] = o;
    var bSpec = Units.buildings[o.data.building.type];
    if(bSpec && bSpec.onDraw) {
      //console.log('updateBuildingGraphics', o.data.building.type);
      bSpec.onDraw(o);
    } else {
      console.log('updateBuildingGraphics handler not found:', o.data.building.type, bSpec);
    }
  },
  
  constructBuildingAt : function(x, y, type) {
    x = Math.floor(x);
    y = Math.floor(y);
    var bSpec = Units.buildings[type];
    if(!bSpec) {
      console.log('ERROR: illegal building type', type);
      return;
    }
    var go = GameMap.getTileGameObjectAt(x, y);
    if(!go) {
      console.log('ERROR: cannot find game object for tile at', x, y);
      return;
    }
    if(Game.state.money < bSpec.cost) {
      UI.plop(x, y, '$$$!', 0xff0000);
      console.log('ERROR: not enough money to build that', type);
      return;
    }
    if(!Units.canPlace(type, go.data)) {
      console.log('cannot place item here');
      return;
    }
    Game.bookMoney(-bSpec.cost, type);
    UI.plop(x, y, '-$'+bSpec.cost+'M');
    Game.state.yikes.buzz += bSpec.cost*Game.state.yikes.buzzFactor;
    UI.message('+'+(bSpec.cost*Game.state.yikes.buzzFactor)+' buzz from new '+bSpec.name, 'lightgreen');    
    if(go.data.building) {
      Units.destroyBuildingOnTile(go);
    }
    go.data.building = {
      type : type,
      progress : 0,
      time : bSpec.time,
      };
    GameMap.draw(go);
    Sound.play('pof');
  },
  
  canPlace : function(buildingType, tile) {
    var bSpec = Units.buildings[buildingType];
    if(bSpec && bSpec.onPlace) {
      return(bSpec.onPlace(tile));
    }
    else {
      return(tile.type == 'ground');
    }
  },
  
  tick : function(deltaTime) {
    each(Units.buildingList, function(o) {
      var bSpec = Units.buildings[o.data.building.type];
      if(bSpec && bSpec.onTick) {
        if(bSpec.interval) {
          bSpec.time += deltaTime;
          if(bSpec.time > bSpec.interval) {
            bSpec.time = 0;
            bSpec.onTick(o);
          }
        } else {
          bSpec.onTick(o);
        }
      }
    });
  },
  
  buildings : {
    
    harbor : {
      name : 'Harbor',
      cost : 5,
      buildTime : 2,
      time : 0,
      interval : 1,
      onTick : function(tileo) {
        if(Game.state.people.length < 100 && Math.random() < 0.03*(Game.state.yikes.rating+Game.state.yikes.buzz)) {
          UI.plop(tileo.data.x, tileo.data.y, '+1');
          People.spawn(tileo.data);
        }
      },
      onPlace : function(tile) {
        return(GameMap.tileIsOnCoast(tile) && tile.type == 'ground');
      },
      onDraw : function(o) {
        var dir = 's';
        if(GameMap.getTile(o.data.x-1, o.data.y).type == 'water')
          dir = 'w';
        if(GameMap.getTile(o.data.x+1, o.data.y).type == 'water')
          dir = 'e';
        if(GameMap.getTile(o.data.x, o.data.y-1).type == 'water')
          dir = 'n';
        var img = new PIXI.Sprite.fromImage('img/ld38-harbor-'+dir+'.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'home',
      onUse : function(tile, person) {
        if(person.wantsToUse == 'harbor') {
          // check-out
          People.leave(person);
          UI.plop(tile.x, tile.y, '-1');
        }
      },
    },

    hotel : {
      name : 'Hotel',
      cost : 3,
      buildTime : 2,
      time : 0,
      onDraw : function(o) {
        var img = new PIXI.Sprite.fromImage('img/ld38-hotel.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'rest',
      onUse : function(tile, person) {
        People.updateNeed(person, 'rest', -1);
        Game.bookMoney(0.01, 'hotel');
      },
    },

    restaurant : {
      name : 'Restaurant',
      cost : 2,
      buildTime : 2,
      time : 0,
      onDraw : function(o) {
        var img = new PIXI.Sprite.fromImage('img/ld38-restaurant.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'hunger',
      onUse : function(tile, person) {
        People.updateNeed(person, 'hunger', -1);
        People.updateNeed(person, 'rest', -0.1);
        Game.bookMoney(0.01, 'restaurant');
      },
    },

    beach : {
      name : 'Beach',
      cost : 1,
      buildTime : 1,
      time : 0,
      onPlace : function(tile) {
        return(GameMap.tileIsOnCoast(tile) && tile.type == 'ground');
      },
      onDraw : function(o) {
        var img = new PIXI.Sprite.fromImage('img/ld38-beach.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'beach',
      onUse : function(tile, person) {
        People.updateNeed(person, 'beach', -1);
        People.updateNeed(person, 'rest', -0.1);
        Game.bookMoney(0.01, 'beach');
      },
    },

    club : {
      name : 'Club',
      cost : 12,
      buildTime : 2,
      time : 0,
      onDraw : function(o) {
        var img = new PIXI.Sprite.fromImage('img/ld38-club.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'party',
      onUse : function(tile, person) {
        People.updateNeed(person, 'party', -1);
        Game.bookMoney(0.2, 'club');
      },
    },

    pool : {
      name : 'Pool',
      cost : 1,
      buildTime : 1,
      time : 0,
      onDraw : function(o) {
        var img = new PIXI.Sprite.fromImage('img/ld38-pool.png');
        img.width = Stage.tileSize;
        img.height = Stage.tileSize;
        o.addChild(img);
      },
      linkedNeed : 'pool',
      onUse : function(tile, person) {
        People.updateNeed(person, 'pool', -1);
        Game.bookMoney(0.02, 'pool');
      },
    },
    
  },
  
}